//+------------------------------------------------------------------+
//|                                               SALChartPoints.mqh |
//|                                         Evgeniy Sergeev (c) 2015 |
//|                                               http://saltrade.ru |
//+------------------------------------------------------------------+
#property copyright "Evgeniy Sergeev (c) 2015"
#property link      "http://saltrade.ru"
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
enum ENUM_POINTS_TYPE
  {
   CLOSE= 0,
   OPEN = 1,
   HIGH = 3,
   LOW=4,
   BODY_HIGH= 5,
   BODY_LOW = 6
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
struct SALPoint
  {
   datetime          time;
   double            value;
   double            high;
   double            low;
   double            open;
   double            close;
   double            body_high;
   double            body_low;
   int               index;
   bool              exists;
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CSALChartPoints
  {
private:
   MqlRates          m_rates[];
   ENUM_POINTS_TYPE  m_defaultPointType;
   ENUM_TIMEFRAMES   m_defaultTimeFrame;

   SALPoint Format(const int index)
     {
      SALPoint dummy={};
      dummy.time = m_rates[index].time;
      dummy.high = m_rates[index].high;
      dummy.low=m_rates[index].low;
      dummy.open=m_rates[index].open;
      dummy.close=m_rates[index].close;

      dummy.body_high= m_rates[index].open>m_rates[index].close ? m_rates[index].open : m_rates[index].close;
      dummy.body_low = m_rates[index].open<m_rates[index].close ? m_rates[index].open : m_rates[index].close;

      switch(m_defaultPointType)
        {
         case CLOSE:
            dummy.value=m_rates[index].close;
            break;
         case HIGH:
            dummy.value=m_rates[index].high;
            break;
         case LOW:
            dummy.value=m_rates[index].low;
            break;

         case BODY_HIGH:
            dummy.value=dummy.body_high;
            break;

         case BODY_LOW:
            dummy.value=dummy.body_low;
            break;

         case OPEN:
            default:
            dummy.value=m_rates[index].open;
        }
      dummy.index=index;
      dummy.exists=true;
      return dummy;
     }
public:
                     CSALChartPoints()
     {
      ArraySetAsSeries(m_rates,true);
      m_defaultTimeFrame=PERIOD_CURRENT;
      PointType(OPEN);
     }

                     CSALChartPoints(ENUM_POINTS_TYPE _type)
     {
      ArraySetAsSeries(m_rates,true);
      m_defaultTimeFrame=PERIOD_CURRENT;
      PointType(_type);
     }

   void PointType(ENUM_POINTS_TYPE _type)
     {
      m_defaultPointType=_type;
     }
   ENUM_POINTS_TYPE PointType()
     {
      return m_defaultPointType;
     }

   void SetTimeframe(ENUM_TIMEFRAMES _timeframe)
     {
      m_defaultTimeFrame=_timeframe;
     }

   int Create(const int _count)
     {
      return CopyRates(Symbol(), m_defaultTimeFrame, 0, _count, m_rates);
     }

   SALPoint At(const int index)
     {
      SALPoint dummy={};
      dummy.exists=false;
      if(index<0 || (ArraySize(m_rates)<=index && Create(index+1)<=index))
         return dummy;
      return Format(index);
     }

   SALPoint At(const datetime _time)
     {
      int index=0;
      for(; At(index).time>_time && At(index).exists; index++)
        {
         // do nothing
        }
      return At(index);
     }

   double Avg(const int _count)
     {

      double sum= 0;
      for(int i = _count; i>0; i--)
         sum+=At(i).high-At(i).low;

      return sum/_count/Point();
     }

   int Total()
     {
      return ArraySize(m_rates);
     }

   CSALChartPoints *Get(ENUM_POINTS_TYPE _type)
     {
      PointType(_type);
      return GetPointer(this);
     }

  };
//+------------------------------------------------------------------+
